"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var defaultConfig = {
    initialText: 'I am a sample text, you can highlight me!',
    startIdx: 0,
    endIdx: 1,
    markerClass: 'marker',
    tokenClass: 'token',
    beginClass: 'begin',
    endClass: 'end',
    selectedClass: 'selected',
    draggedClass: 'dragged'
};
var SelectionMarker = /** @class */ (function () {
    function SelectionMarker(el, config) {
        this.el = el;
        this.config = config;
        this.config = Object.assign({}, defaultConfig, this.config);
        this.initContainer();
        this.createTokens();
        this.initDragEvents();
    }
    SelectionMarker.prototype.initContainer = function () {
        this.el.style.display = 'flex';
        this.el.style.userSelect = 'none';
        this.begin = this.createMarker(true, this.config.startIdx * 2);
        this.end = this.createMarker(false, this.config.endIdx * 2);
        this.el.appendChild(this.begin.node);
        this.el.appendChild(this.end.node);
    };
    SelectionMarker.prototype.createTokens = function () {
        var _this = this;
        this.el.innerHTML = this.config.initialText
            .split(' ')
            .map(function (t, idx) {
            var order = idx * 2 + 1;
            var selectedClass = (idx >= _this.config.startIdx && idx < _this.config.endIdx)
                ? _this.config.selectedClass : '';
            return "<div class=\"" + _this.config.tokenClass + " " + selectedClass + "\" data-order=" + order + " style=\"order:" + order + "\">" +
                (t + "</div>");
        })
            .join('');
    };
    SelectionMarker.prototype.initDragEvents = function () {
        var _this = this;
        this.el.addEventListener('dragstart', function (e) {
            var target = e.target;
            if (target.classList.contains(_this.config.markerClass)) {
                _this.dragged = (target.id === _this.config.beginClass) ? _this.begin : _this.end;
                setTimeout(function () { return target.classList.add(_this.config.draggedClass); });
            }
        });
        this.el.addEventListener('dragenter', function (e) {
            var target = e.target;
            if (_this.dragged && _this.isToken(target)) {
                var order = Number.parseInt(target.dataset.order);
                if (_this.dragged === _this.begin && order + 1 <= _this.end.order) {
                    _this.updateSelection(_this.dragged, order - 1, target);
                }
                else if (_this.dragged === _this.end && order - 1 >= _this.begin.order) {
                    _this.updateSelection(_this.dragged, order + 1, target);
                }
            }
        });
        this.el.addEventListener('dragend', function (e) {
            var target = e.target;
            target.classList.remove(_this.config.draggedClass);
            _this.dragged = null;
        });
    };
    SelectionMarker.prototype.isToken = function (target) {
        return target.classList.contains(this.config.tokenClass);
    };
    SelectionMarker.prototype.updateSelection = function (dragged, order, target) {
        var _this = this;
        dragged.node.style.order = '' + order;
        dragged.order = order;
        var startIdx = this.begin.order / 2;
        var endIdx = this.end.order / 2;
        var selectedText = '';
        this.el.children.slice().forEach(function (child, idx) {
            if (idx >= startIdx && idx < endIdx) {
                child.classList.add(_this.config.selectedClass);
                selectedText += child.textContent;
            }
            else {
                child.classList.remove(_this.config.selectedClass);
            }
        });
        this.config.onMarked(selectedText, startIdx, endIdx);
    };
    SelectionMarker.prototype.createMarker = function (isBegin, order) {
        if (isBegin === void 0) { isBegin = false; }
        var container = document.createElement('div');
        var id = (isBegin ? this.config.beginClass : this.config.endClass);
        container.innerHTML = "<div draggable=true class=\"" + this.config.markerClass + " " + id + "\"" +
            (" id=" + id + " style=\"order:" + order + "\"></div>");
        var node = container.firstElementChild;
        return {
            node: node,
            order: order,
            isBegin: isBegin
        };
    };
    return SelectionMarker;
}());
exports.SelectionMarker = SelectionMarker;
//# sourceMappingURL=index.js.map